@@ -21,8 +21,6 @@
 
 typedef struct ThreadSpecificData {
     char	    	nabuf[16];
-    struct tm   	gtbuf;
-    struct tm   	ltbuf;
     struct {
 	Tcl_DirEntry ent;
 	char name[MAXNAMLEN+1];
@@ -68,7 +66,7 @@ static pthread_mutex_t *allocLockPtr = &allocLock;
 /*
  *----------------------------------------------------------------------
  *
- * Tcl_CreateThread --
+ * TclpThreadCreate --
  *
  *	This procedure creates a new thread.
  *
@@ -83,7 +81,7 @@ static pthread_mutex_t *allocLockPtr = &allocLock;
  */
 
 int
-Tcl_CreateThread(idPtr, proc, clientData, stackSize, flags)
+TclpThreadCreate(idPtr, proc, clientData, stackSize, flags)
     Tcl_ThreadId *idPtr;		/* Return, the ID of the thread */
     Tcl_ThreadCreateProc proc;		/* Main() function of the thread */
     ClientData clientData;		/* The one argument to Main() */
@@ -161,16 +159,16 @@ Tcl_CreateThread(idPtr, proc, clientData, stackSize, flags)
  */
 
 int
-Tcl_JoinThread(id, state)
-    Tcl_ThreadId id;	/* Id of the thread to wait upon */
-    int*     state;	/* Reference to the storage the result
-			 * of the thread we wait upon will be
-			 * written into. */
+Tcl_JoinThread(threadId, state)
+    Tcl_ThreadId threadId; /* Id of the thread to wait upon */
+    int*     state;	   /* Reference to the storage the result
+			    * of the thread we wait upon will be
+			    * written into. */
 {
 #ifdef TCL_THREADS
     int result;
 
-    result = pthread_join ((pthread_t) id, (VOID**) state);
+    result = pthread_join ((pthread_t) threadId, (VOID**) state);
     return (result == 0) ? TCL_OK : TCL_ERROR;
 #else
     return TCL_ERROR;
@@ -255,7 +253,37 @@ TclpInitLock()
     pthread_mutex_lock(&initLock);
 #endif
 }
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TclpFinalizeLock
+ *
+ *	This procedure is used to destroy all private resources used in
+ *	this file.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Destroys everything private.  TclpInitLock must be held
+ *	entering this function.
+ *
+ *----------------------------------------------------------------------
+ */
 
+void
+TclFinalizeLock ()
+{
+#ifdef TCL_THREADS
+    /*
+     * You do not need to destroy mutexes that were created with the
+     * PTHREAD_MUTEX_INITIALIZER macro.  These mutexes do not need
+     * any destruction: masterLock, allocLock, and initLock.
+     */
+    pthread_mutex_unlock(&initLock);
+#endif
+}
 
 /*
  *----------------------------------------------------------------------
@@ -466,6 +494,7 @@ TclpFinalizeMutex(mutexPtr)
 {
     pthread_mutex_t *pmutexPtr = *(pthread_mutex_t **)mutexPtr;
     if (pmutexPtr != NULL) {
+        pthread_mutex_destroy(pmutexPtr);
 	ckfree((char *)pmutexPtr);
 	*mutexPtr = NULL;
     }
@@ -819,7 +848,7 @@ TclpReaddir(DIR * dir)
 #   endif /* HAVE_STRUCT_DIRENT64 */
     if (ent != NULL) {
 	memcpy((VOID *) &tsdPtr->rdbuf.ent, (VOID *) ent,
-		sizeof(&tsdPtr->rdbuf));
+		sizeof(tsdPtr->rdbuf));
 	ent = &tsdPtr->rdbuf.ent;
     }
     Tcl_MutexUnlock(&rdMutex);
@@ -834,51 +863,6 @@ TclpReaddir(DIR * dir)
 #endif
     return ent;
 }
-
-#if defined(TCL_THREADS) && (!defined(HAVE_GMTIME_R) || !defined(HAVE_LOCALTIME_R))
-TCL_DECLARE_MUTEX( tmMutex )
-#undef localtime
-#undef gmtime
-#endif
-
-struct tm *
-TclpLocaltime(time_t * clock)
-{
-#ifdef TCL_THREADS
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-
-#ifdef HAVE_LOCALTIME_R
-    return localtime_r(clock, &tsdPtr->ltbuf);
-#else
-    Tcl_MutexLock( &tmMutex );
-    memcpy( (VOID *) &tsdPtr->ltbuf, (VOID *) localtime( clock ), sizeof (struct tm) );
-    Tcl_MutexUnlock( &tmMutex );
-    return &tsdPtr->ltbuf;
-#endif    
-#else
-    return localtime(clock);
-#endif
-}
-
-struct tm *
-TclpGmtime(time_t * clock)
-{
-#ifdef TCL_THREADS
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-
-#ifdef HAVE_GMTIME_R
-    return gmtime_r(clock, &tsdPtr->gtbuf);
-#else
-    Tcl_MutexLock( &tmMutex );
-    memcpy( (VOID *) &tsdPtr->gtbuf, (VOID *) gmtime( clock ), sizeof (struct tm) );
-    Tcl_MutexUnlock( &tmMutex );
-    return &tsdPtr->gtbuf;
-#endif    
-#else
-    return gmtime(clock);
-#endif
-}
-
 char *
 TclpInetNtoa(struct in_addr addr)
 {
@@ -902,19 +886,20 @@ TclpInetNtoa(struct in_addr addr)
  * Additions by AOL for specialized thread memory allocator.
  */
 #ifdef USE_THREAD_ALLOC
-static int initialized = 0;
+static volatile int initialized = 0;
 static pthread_key_t	key;
-static pthread_once_t	once = PTHREAD_ONCE_INIT;
+
+typedef struct allocMutex {
+    Tcl_Mutex       tlock;
+    pthread_mutex_t plock;
+} allocMutex;
 
 Tcl_Mutex *
 TclpNewAllocMutex(void)
 {
-    struct lock {
-        Tcl_Mutex       tlock;
-        pthread_mutex_t plock;
-    } *lockPtr;
+    struct allocMutex *lockPtr;
 
-    lockPtr = malloc(sizeof(struct lock));
+    lockPtr = malloc(sizeof(struct allocMutex));
     if (lockPtr == NULL) {
 	panic("could not allocate lock");
     }
@@ -923,20 +908,41 @@ TclpNewAllocMutex(void)
     return &lockPtr->tlock;
 }
 
-static void
-InitKey(void)
+void
+TclpFreeAllocMutex(mutex)
+    Tcl_Mutex *mutex; /* The alloc mutex to free. */
+{
+    allocMutex* lockPtr = (allocMutex*) mutex;
+    if (!lockPtr) return;
+    pthread_mutex_destroy(&lockPtr->plock);
+    free(lockPtr);
+}
+
+void TclpFreeAllocCache(ptr)
+    void *ptr;
 {
     extern void TclFreeAllocCache(void *);
 
-    pthread_key_create(&key, TclFreeAllocCache);
-    initialized = 1;
+    TclFreeAllocCache(ptr);
+    /*
+     * Perform proper cleanup of things done in TclpGetAllocCache.
+     */
+    if (initialized) {
+        pthread_key_delete(key);
+        initialized = 0;
+    }
 }
 
 void *
 TclpGetAllocCache(void)
 {
     if (!initialized) {
-	pthread_once(&once, InitKey);
+	pthread_mutex_lock(allocLockPtr);
+	if (!initialized) {
+	    pthread_key_create(&key, TclpFreeAllocCache);
+	    initialized = 1;
+	}
+	pthread_mutex_unlock(allocLockPtr);
     }
     return pthread_getspecific(key);
 }
