@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinFile.c,v 1.44.2.6 2003/10/03 17:45:37 vincentdarley Exp $
+ * RCS: @(#) $Id: tclWinFile.c,v 1.44.2.9 2004/07/02 16:52:20 vincentdarley Exp $
  */
 
 //#define _WIN32_WINNT  0x0500
@@ -665,12 +665,11 @@ NativeWriteReparse(LinkDirectory, buffer)
  *	application, given its argv[0] value.
  *
  * Results:
- *	A dirty UTF string that is the path to the executable.  At this
- *	point we may not know the system encoding.  Convert the native
- *	string value to UTF using the default encoding.  The assumption
- *	is that we will still be able to parse the path given the path
- *	name contains ASCII string and '/' chars do not conflict with
- *	other UTF chars.
+ *	A clean UTF string that is the path to the executable.  At this
+ *	point we may not know the system encoding, but we convert the
+ *	string value to UTF-8 using core Windows functions.  The path name
+ *	contains ASCII string and '/' chars do not conflict with other UTF
+ *	chars.
  *
  * Side effects:
  *	The variable tclNativeExecutableName gets filled in with the file
@@ -685,8 +684,8 @@ TclpFindExecutable(argv0)
     CONST char *argv0;		/* The value of the application's argv[0]
 				 * (native). */
 {
-    Tcl_DString ds;
     WCHAR wName[MAX_PATH];
+    char name[MAX_PATH * TCL_UTF_MAX];
 
     if (argv0 == NULL) {
 	return NULL;
@@ -700,12 +699,15 @@ TclpFindExecutable(argv0)
      * create this process.
      */
 
-    (*tclWinProcs->getModuleFileNameProc)(NULL, wName, MAX_PATH);
-    Tcl_WinTCharToUtf((CONST TCHAR *) wName, -1, &ds);
+    if (GetModuleFileNameW(NULL, wName, MAX_PATH) == 0) {
+	GetModuleFileNameA(NULL, name, sizeof(name));
+    } else {
+	WideCharToMultiByte(CP_UTF8, 0, wName, -1, 
+		name, sizeof(name), NULL, NULL);
+    }
 
-    tclNativeExecutableName = ckalloc((unsigned) (Tcl_DStringLength(&ds) + 1));
-    strcpy(tclNativeExecutableName, Tcl_DStringValue(&ds));
-    Tcl_DStringFree(&ds);
+    tclNativeExecutableName = ckalloc((unsigned) (strlen(name) + 1));
+    strcpy(tclNativeExecutableName, name);
 
     TclWinNoBackslash(tclNativeExecutableName);
     return tclNativeExecutableName;
@@ -842,11 +844,11 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 	dirName = Tcl_DStringAppend(&dirString, "*.*", 3);
 	native = Tcl_WinUtfToTChar(dirName, -1, &ds);
 	handle = (*tclWinProcs->findFirstFileProc)(native, &data);
-	Tcl_DStringFree(&ds);
 
 	if (handle == INVALID_HANDLE_VALUE) {
-	    Tcl_DStringFree(&dirString);
 	    TclWinConvertError(GetLastError());
+	    Tcl_DStringFree(&ds);
+	    Tcl_DStringFree(&dirString);
 	    Tcl_ResetResult(interp);
 	    Tcl_AppendResult(interp, "couldn't read directory \"",
 		    Tcl_DStringValue(&dsOrig), "\": ", 
@@ -854,6 +856,7 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 	    Tcl_DStringFree(&dsOrig);
 	    return TCL_ERROR;
 	}
+	Tcl_DStringFree(&ds);
 
 	/*
 	 * Check to see if the pattern should match the special
@@ -2260,26 +2263,55 @@ TclpObjNormalizePath(interp, pathPtr, nextCheckpoint)
 		    }
 		    Tcl_DStringAppend(&dsNorm,nativePath,Tcl_DStringLength(&ds));
 		} else {
-		    WIN32_FIND_DATAW fData;
-		    HANDLE handle;
+		    char *checkDots = NULL;
 		    
-		    handle = FindFirstFileW((WCHAR*)nativePath, &fData);
-		    if (handle == INVALID_HANDLE_VALUE) {
-			/* This is usually the '/' in 'c:/' at end of string */
-			Tcl_DStringAppend(&dsNorm,(CONST char*)L"/", 
-					  sizeof(WCHAR));
+		    if (lastValidPathEnd[1] == '.') {
+			checkDots = lastValidPathEnd + 1;
+			while (checkDots < currentPathEndPosition) {
+			    if (*checkDots != '.') {
+				checkDots = NULL;
+				break;
+			    }
+			    checkDots++;
+			}
+		    }
+		    if (checkDots != NULL) {
+			int dotLen = currentPathEndPosition - lastValidPathEnd;
+			/* 
+			 * Path is just dots.  We shouldn't really
+			 * ever see a path like that.  However, to be
+			 * nice we at least don't mangle the path -- 
+			 * we just add the dots as a path segment and
+			 * continue
+			 */
+			Tcl_DStringAppend(&dsNorm,
+					  (TCHAR*)((WCHAR*)(nativePath 
+						+ Tcl_DStringLength(&ds)) 
+						- dotLen),
+					  (int)(dotLen * sizeof(WCHAR)));
 		    } else {
-			WCHAR *nativeName;
-			if (fData.cFileName[0] != '\0') {
-			    nativeName = fData.cFileName;
+			/* Normal path */
+			WIN32_FIND_DATAW fData;
+			HANDLE handle;
+			
+			handle = FindFirstFileW((WCHAR*)nativePath, &fData);
+			if (handle == INVALID_HANDLE_VALUE) {
+			    /* This is usually the '/' in 'c:/' at end of string */
+			    Tcl_DStringAppend(&dsNorm,(CONST char*)L"/", 
+					      sizeof(WCHAR));
 			} else {
-			    nativeName = fData.cAlternateFileName;
+			    WCHAR *nativeName;
+			    if (fData.cFileName[0] != '\0') {
+				nativeName = fData.cFileName;
+			    } else {
+				nativeName = fData.cAlternateFileName;
+			    }
+			    FindClose(handle);
+			    Tcl_DStringAppend(&dsNorm,(CONST char*)L"/", 
+					      sizeof(WCHAR));
+			    Tcl_DStringAppend(&dsNorm,(TCHAR*)nativeName, 
+					      (int) (wcslen(nativeName)*sizeof(WCHAR)));
 			}
-			FindClose(handle);
-			Tcl_DStringAppend(&dsNorm,(CONST char*)L"/", 
-					  sizeof(WCHAR));
-			Tcl_DStringAppend(&dsNorm,(TCHAR*)nativeName, 
-					  (int) (wcslen(nativeName)*sizeof(WCHAR)));
 		    }
 		}
 		Tcl_DStringFree(&ds);
